@@ -70,6 +70,9 @@ func TestCSS(t *testing.T) {
 		{"{d:url( \n  \n\t0", "{d:url()}"},
 		{"{d:urL(     '0", `{d:url("'")}`},
 		{`{-ms-filter:"`, `{-ms-filter:"}`},
+		{"@import url(", `@import url(`},
+		{"@import url( ", `@import url( `},
+		{"@import url(  )", `@import ""`},
 	}
 
 	m := minify.New()
@@ -145,6 +148,7 @@ func TestCSSInline(t *testing.T) {
 		{"background-position:bottom left", "background-position:0 100%"},
 		{"background-position:top center", "background-position:50% 0"},
 		{"background-position:bottom center", "background-position:50% 100%"},
+		{"background-position:center left", "background-position:0"},
 		{"background-position:bottom 5% right 0%", "background-position:100% 95%"},
 		{"background-position:bottom 0 right 10%", "background-position:90% 100%"},
 		{"background-position:top 10% left 5%", "background-position:5% 10%"},
@@ -259,6 +263,7 @@ func TestCSSInline(t *testing.T) {
 		{"flex:5 0 0%", "flex:5 0"},
 		{"flex:5 0 0px", "flex:5 0"},
 		{"flex:0 1 auto", "flex:initial"},
+		{"flex:0 5 auto", "flex:0 5 auto"},
 		{"flex:1 1 auto", "flex:auto"},
 		{"flex:0 0 auto", "flex:none"},
 		{"flex:3 1 auto", "flex:3 1 auto"},
