@@ -42,7 +42,7 @@ func TestHTML(t *testing.T) {
 		{`<span attr="test"></span>`, `<span attr=test></span>`},
 		{`<span attr='test&apos;test'></span>`, `<span attr="test'test"></span>`},
 		{`<span attr="test&quot;test"></span>`, `<span attr='test"test'></span>`},
-		{`<span attr='test""&apos;&amp;test'></span>`, `<span attr='test""&#39;&test'></span>`},
+		{`<span attr='test""&apos;&amp;test'></span>`, `<span attr='test""&#39;&amp;test'></span>`},
 		{`<span attr="test/test"></span>`, `<span attr=test/test></span>`},
 		{`<span attr="test/"></span>`, `<span attr=test/></span>`},
 		{`<span>&amp;</span>`, `<span>&</span>`},
@@ -151,18 +151,20 @@ func TestHTML(t *testing.T) {
 		{`<script><!--<`, `<script><!--<`},
 
 		// bugs
-		{`<svg id="1"></svg>`, `<svg id="1"></svg>`},                                                             // #67
-		{`<pre> <x> a  b </x> </pre>`, `<pre> <x> a  b </x> </pre>`},                                             // #82
-		{`text <progress></progress> text`, `text <progress></progress> text`},                                   // #89
-		{`text <img> text`, `text <img> text`},                                                                   // #89
-		{`<p>text</p><br>text`, `<p>text</p><br>text`},                                                           // #122
-		{`<amp-analytics type=adobeanalytics_nativeConfig>`, `<amp-analytics type=adobeanalytics_nativeConfig>`}, // #270
-		{`<input title=""><input lang="">`, `<input title><input lang>`},                                         // #331
-		{`<span translate="no">`, `<span translate=no>`},                                                         // #352
-		{`<i class="fas"> </i> Text`, `<i class=fas></i>Text`},                                                   // #390
-		{`<i class="fas"></i> Text`, `<i class=fas></i> Text`},                                                   // #390
-		{`a <span></span> b`, `a <span></span> b`},                                                               // #427
-		{`<canvas><p>test</p></canvas>`, `<canvas><p>test</p></canvas>`},                                         // #440
+		{`<svg id="1"></svg>`, `<svg id="1"></svg>`},                                                              // #67
+		{`<pre> <x> a  b </x> </pre>`, `<pre> <x> a  b </x> </pre>`},                                              // #82
+		{`text <progress></progress> text`, `text <progress></progress> text`},                                    // #89
+		{`text <img> text`, `text <img> text`},                                                                    // #89
+		{`<p>text</p><br>text`, `<p>text</p><br>text`},                                                            // #122
+		{`<amp-analytics type=adobeanalytics_nativeConfig>`, `<amp-analytics type=adobeanalytics_nativeConfig>`},  // #270
+		{`<input title=""><input lang="">`, `<input title><input lang>`},                                          // #331
+		{`<span translate="no">`, `<span translate=no>`},                                                          // #352
+		{`<i class="fas"> </i> Text`, `<i class=fas></i>Text`},                                                    // #390
+		{`<i class="fas"></i> Text`, `<i class=fas></i> Text`},                                                    // #390
+		{`a <span></span> b`, `a <span></span> b`},                                                                // #427
+		{`<canvas><p>test</p></canvas>`, `<canvas><p>test</p></canvas>`},                                          // #440
+		{`<source type='video/mp4; codecs="av01.0.05M.08"'>`, `<source type='video/mp4;codecs="av01.0.05M.08"'>`}, // #545
+		{`<html lang="mn-Cyrl-MN">`, `<html lang=mn-Cyrl-MN>`},                                                    // #579
 	}
 
 	m := minify.New()
