@@ -23,7 +23,11 @@ type Minifier struct {
 	Precision           int // number of significant digits
 	KeepVarNames        bool
 	useAlphabetVarNames bool
-	NoNullishOperator   bool
+	Version             int
+}
+
+func (o *Minifier) minVersion(version int) bool {
+	return o.Version == 0 || version <= o.Version
 }
 
 // Minify minifies JS data, it reads from r and writes to w.
@@ -303,7 +307,7 @@ func (m *jsMinifier) minifyStmt(i js.IStmt) {
 		if stmt.Catch != nil {
 			m.write(catchBytes)
 			stmt.Catch.List = optimizeStmtList(stmt.Catch.List, defaultBlock)
-			if v, ok := stmt.Binding.(*js.Var); ok && v.Uses == 1 {
+			if v, ok := stmt.Binding.(*js.Var); ok && v.Uses == 1 && m.o.minVersion(2019) {
 				stmt.Catch.Scope.Declared = stmt.Catch.Scope.Declared[1:]
 				stmt.Binding = nil
 			}
@@ -1139,10 +1143,10 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 		parentInFor := m.inFor
 		m.inFor = false
 		for _, item := range expr.List {
-			m.write(item.Value)
+			m.write(replaceEscapes(item.Value, '`', 1, 2))
 			m.minifyExpr(item.Expr, js.OpExpr)
 		}
-		m.write(expr.Tail)
+		m.write(replaceEscapes(expr.Tail, '`', 1, 1))
 		m.inFor = parentInFor
 	case *js.NewExpr:
 		if expr.Args == nil && js.OpLHS < prec && prec != js.OpNew {
